from sidecar.app_instance_event_handler import AppInstanceEventHandler
from sidecar.app_instance_events import AppInstanceEvents
from sidecar.app_instance_identifier import AppInstanceIdentifier
from sidecar.const import AppInstanceConfigStatus
from logging import Logger


class AppInstanceConfigStatusEventReporter:
    def __init__(self, app_instance_event_handler: AppInstanceEventHandler, logger: Logger) -> None:
        super().__init__()
        self.logger = logger
        self._app_instance_event_handler = app_instance_event_handler

    def on_app_instance_configuration_status_updated(self, app_instance_identifier: AppInstanceIdentifier,
                                                     app_instance_config_status: str):
        app_instance_event = self._convert_to_app_instance_event(app_instance_config_status, logger=self.logger)
        self._app_instance_event_handler.report_event(app_instance_identifier, app_instance_event)

    @staticmethod
    def _convert_to_app_instance_event(app_instance_config_status: str, logger: Logger) -> str:
        if app_instance_config_status == AppInstanceConfigStatus.PENDING:
            return AppInstanceEvents.RunningHealthCheck
        if app_instance_config_status == AppInstanceConfigStatus.COMPLETED:
            return AppInstanceEvents.DeploymentCompleted
        if app_instance_config_status == AppInstanceConfigStatus.ERROR:
            return AppInstanceEvents.DeploymentFailed
        raise Exception("unknown AppInstanceConfigStatus '{status}'".format(status=app_instance_config_status))



from queue import Queue, Empty
from threading import Thread

import time


class NonBlockingStreamReader:

    def __init__(self, stream, interval=1):

        self._s = stream
        self._q = Queue()
        self.to_break = False
        self.interval = interval

        def populate_queue(stream_in, queue):

            while True:
                try:
                    line = stream_in.readline()
                    if line:
                        queue.put(line)
                    if self.to_break:
                        break
                    time.sleep(1)
                except:
                    pass

        self._t = Thread(target=populate_queue,
                         args=(self._s, self._q))
        self._t.daemon = True
        self._t.start()  # start collecting lines from the stream

    def stop(self):
        self.to_break = True

    def read_line(self, timeout=None):
        try:
            return self._q.get(block=timeout is not None,
                               timeout=timeout)
        except Empty:
            return None


class UnexpectedEndOfStream(Exception):
    pass



import threading

from sidecar.apps_configuration_end_tracker import AppConfigurationEndStatus
from sidecar.health_check.app_health_check_state import AppHealthCheckState
from sidecar.messaging_service import MessagingService
from sidecar.sandbox_deployment_state_tracker import SandboxDeploymentStateTracker


class SandboxEndDeploymentNotifier:
    def __init__(self,
                 deployment_state_tracker: SandboxDeploymentStateTracker,
                 messaging_service: MessagingService,
                 app_health_state_checker: AppHealthCheckState,
                 space_id: str,
                 sandbox_id: str):
        self._sandbox_id = sandbox_id
        self._space_id = space_id
        self._app_health_state_checker = app_health_state_checker
        self._messaging_service = messaging_service
        self._deployment_state_tracker = deployment_state_tracker
        self._is_message_sent = False
        self._lock = threading.RLock()

    # please never use git history on this, it's not my fault.
    def notify_end_deployment(self):

        with self._lock:
            if self._is_message_sent:
                return

            if self._deployment_state_tracker.all_apps_deployment_ended_with_status(AppConfigurationEndStatus.COMPLETED) \
                    and self._app_health_state_checker.all_complete_with_success():

                end_status = self._deployment_state_tracker.get_deployment_end_status()

                message = {
                    'SpaceId': self._space_id,
                    'SandboxId': self._sandbox_id,
                    'Status': end_status
                }

                self._messaging_service.publish('CSMSCommon.Model.Events:SandboxDeployed', message)
                self._is_message_sent = True



from azure.mgmt.compute import ComputeManagementClient
# from azure.mgmt.resource import ResourceManagementClient
# from azure.mgmt.resource import SubscriptionClient
from azure.mgmt.cosmosdb import CosmosDB
# from azure.mgmt.storage import StorageManagementClient
from msrestazure.azure_active_directory import ServicePrincipalCredentials


class AzureCredentialsProvider:

    def __init__(self, management_resource_group: str, subscription_id: str, application_id: str,
                 application_secret: str, tenant_id: str):
        self.management_resource_group = management_resource_group
        self.subscription_id = subscription_id
        self.application_id = application_id
        self.application_key = application_secret
        self.tenant = tenant_id
        # self.region = region

    def get_service_principal(self):
        return ServicePrincipalCredentials(client_id=self.application_id,
                                           secret=self.application_key,
                                           tenant=self.tenant)


class AzureClientsManager:

    def __init__(self, credentials_provider: AzureCredentialsProvider):
        self._credentials_provider = credentials_provider
        self._service_principal = self._credentials_provider.get_service_principal()
        # self._resource_client = None
        self._cosmos_client = None
        # self._subscription_client = None
        # self._storage_client = None
        self._compute_client = None
        # self._network_client = None

    @property
    def management_resource_group(self):
        return self._credentials_provider.management_resource_group

    # @property
    # def region(self):
    #     return self._credentials_provider.region

    # @property
    # def resource_client(self):
    #     if self._resource_client is None:
    #         self._resource_client = ResourceManagementClient(self._service_principal,
    #                                                          self._credentials_provider.subscription_id)
    #     return self._resource_client

    # @property
    # def storage_client(self):
    #     if self._storage_client is None:
    #         self._storage_client = StorageManagementClient(self._service_principal,
    #                                                        self._credentials_provider.subscription_id)
    #
    #     return self._storage_client

    # @property
    # def subscription_client(self):
    #     if self._subscription_client is None:
    #         self._subscription_client = SubscriptionClient(self._service_principal)
    #     return self._subscription_client

    @property
    def cosmos_db_client(self):
        if self._cosmos_client is None:
            self._cosmos_client = CosmosDB(self._service_principal, self._credentials_provider.subscription_id)
        return self._cosmos_client

    @property
    def compute_client(self):
        if self._compute_client is None:
            self._compute_client = ComputeManagementClient(self._service_principal,
                                                           self._credentials_provider.subscription_id)
        return self._compute_client

    # @property
    # def network_client(self):
    #     if self._network_client is None:
    #         self._network_client = NetworkManagementClient(self._service_credentials, self._subscription_id)
    #     return self._network_client

    # @property
    # def storage_client(self):
    #     if self._storage_client is None:
    #         self._storage_client = StorageManagementClient(self._service_credentials, self._subscription_id)
    #     return self._storage_client